@@ -2,11 +2,10 @@
 #'
 #' This function extracts the cluster variable from a table by applying a schema
 #' description to it.
-#' @param schema [\code{character(1)}]\cr the schema description of
+#' @param schema [\code{character(1)}]\cr the (validated) schema description of
 #'   \code{input}.
 #' @param input [\code{character(1)}]\cr table to reorganise.
-#' @return list of the length of number of clusters with values cluster
-#'   variable.
+#' @return a list per cluster with values of the cluster variable
 #' @examples
 #' input <- tabs2shift$clusters_nested
 #' schema <- setCluster(id = "sublevel",
@@ -21,11 +20,14 @@
 #'
 #' validateSchema(schema = schema, input = input) %>%
 #'    getClusterVar(input = input)
+#' @importFrom checkmate assertTRUE
 #' @importFrom purrr map set_names
 #' @export
 
 getClusterVar <- function(schema = NULL, input = NULL){
 
+  assertTRUE(x = schema@validated)
+
   clusters <- schema@clusters
   nClusters <- max(lengths(clusters))
 
@@ -59,13 +61,13 @@ getClusterVar <- function(schema = NULL, input = NULL){
             theRows <- theVar$row[ix]
           } else {
             theRows <- c(clusters$row[ix]:(clusters$height[ix]+clusters$row[ix]-1))
-            theRows <- theRows[!theRows %in% filter$row]
+            theRows <- theRows[theRows %in% filter$row]
           }
           if(!is.null(theVar$col[ix])){
             theCols <- theVar$col[ix]
           } else {
             thecol <- c(clusters$col[ix]:(clusters$width[ix]+clusters$col[ix]-1))
-            theCols <- theCols[!theCols %in% filter$col]
+            theCols <- theCols[theCols %in% filter$col]
           }
           input[theRows, theCols]
         })
