@@ -2,11 +2,10 @@
 #'
 #' This function extracts the observed variables from a table by applying a
 #' schema description to it.
-#' @param schema [\code{character(1)}]\cr the schema description of
+#' @param schema [\code{character(1)}]\cr the (validated) schema description of
 #'   \code{input}.
 #' @param input [\code{character(1)}]\cr table to reorganise.
-#' @return list of the length of number of clusters with values of the observed
-#'   variables per cluster
+#' @return a list per cluster with values of the observed variables
 #' @examples
 #' input <- tabs2shift$clusters_nested
 #' schema <- setCluster(id = "sublevel",
@@ -21,12 +20,15 @@
 #'
 #' validateSchema(schema = schema, input = input) %>%
 #'    getObsVars(input = input)
-#' @importFrom purrr map set_names
-#' @importFrom dplyr row_number
+#' @importFrom checkmate assertTRUE
+#' @importFrom purrr map set_names map_chr reduce
+#' @importFrom dplyr row_number filter
 #' @export
 
 getObsVars <- function(schema = NULL, input = NULL){
 
+  assertTRUE(x = schema@validated)
+
   clusters <- schema@clusters
   nClusters <- max(lengths(clusters))
 
@@ -45,7 +47,9 @@ getObsVars <- function(schema = NULL, input = NULL){
     theVar <- variables[[ix]]
     if(theVar$type == "observed"){
       if(is.numeric(theVar$key)){
-        c(theVar$key, theVar$col)
+        if(!any(0 %in% theVar$key)){
+          c(theVar$key, theVar$col)
+        }
       }
     }
   })
@@ -64,11 +68,19 @@ getObsVars <- function(schema = NULL, input = NULL){
         }
         temp <- input[varRows, listedCols]
         names(temp)[1] <- "key"
-        theFilter <- which(varRows %in% filter$row)
+        rowFilter <- which(varRows %in% filter$row)
+        colFilter <- NULL
 
-        if(!is.null(theFilter)){
+        # apply a row filter ...
+        if(!is.null(rowFilter)){
           temp <- temp %>%
-            filter(!row_number() %in% theFilter)
+            filter(row_number() %in% rowFilter)
+        }
+
+        # ... and column filter
+        if(!is.null(colFilter)){
+          temp <- temp %>%
+            select(all_of(colFilter))
         }
 
         # replace keys with their variable name
@@ -96,31 +108,52 @@ getObsVars <- function(schema = NULL, input = NULL){
               } else {
                 temp <- input[varRows, tempVar$col]
               }
-              theFilter <- which(varRows %in% filter$row)
+              rowFilter <- which(varRows %in% filter$row)
+              colFilter <- NULL
             } else if(is.numeric(tempVar$key)){
               temp <- input[varRows, tempVar$col]
-              theFilter <- NULL
+              if(!tempVar$key == 0){
+                rowFilter <- NULL
+                colFilter <- NULL
+              } else {
+                rowFilter <- which(varRows %in% filter$row)
+                colFilter <- NULL
+              }
             }
           } else {
 
             if(!is.null(tempVar$row[ix])){
               if(nClusters != 1){
                 temp <- input[varRows, tempVar$col[ix]]
-                theFilter <- which(varRows %in% filter$row)
+                rowFilter <- which(varRows %in% filter$row)
+                colFilter <- NULL
               } else {
                 temp <- input[varRows, tempVar$col]
-                theFilter <- which(varRows %in% filter$row)
+                rowFilter <- which(varRows %in% filter$row)
+                if(!is.null(filter$col)){
+                  colFilter <- colnames(temp)[tempVar$col %in% filter$col]
+                } else{
+                  colFilter <- NULL
+                }
               }
             } else {
               temp <- input[varRows, tempVar$col[ix]]
-              theFilter <- which(varRows %in% filter$row)
+              rowFilter <- which(varRows %in% filter$row)
+              colFilter <- NULL
             }
 
           }
 
-          if(!is.null(theFilter)){
+          # apply a row filter ...
+          if(!is.null(rowFilter)){
             temp <- temp %>%
-              filter(!row_number() %in% theFilter)
+              filter(row_number() %in% rowFilter)
+          }
+
+          # ... and column filter
+          if(!is.null(colFilter)){
+            temp <- temp %>%
+              select(all_of(colFilter))
           }
 
           vars <- c(vars, set_names(x = list(temp), nm = names(obsVars)[i]))
