@@ -43,11 +43,13 @@ setCluster <- function(schema = NULL, id = NULL, group = NULL, member = NULL,
   assertCharacter(x = id, len = 1, any.missing = FALSE)
   assertCharacter(x = group, len = 1, any.missing = FALSE, null.ok = TRUE)
   colInt <- testIntegerish(x = left, lower = 1, min.len = 1, null.ok = TRUE)
-  colList <- testList(x = left, len = 3)
+  colList <- testList(x = left, len = 1)
   assert(colInt, colList)
+  if(colList) assertSubset(x = names(left), choices = c("find"))
   rowInt <- testIntegerish(x = top, lower = 1, min.len = 1, null.ok = TRUE)
-  rowList <- testList(x = top, len = 3)
+  rowList <- testList(x = top, len = 1)
   assert(rowInt, rowList)
+  if(rowList) assertSubset(x = names(top), choices = c("find"))
   assertIntegerish(x = width, null.ok = TRUE)
   assertIntegerish(x = height, null.ok = TRUE)
   assertIntegerish(x = member, null.ok = TRUE)
