@@ -46,11 +46,13 @@ setIDVar <- function(schema = NULL, name = NULL, value = NULL, columns = NULL,
   assertClass(x = schema, classes = "schema", null.ok = TRUE)
   assertCharacter(x = name, len = 1, any.missing = FALSE)
   colInt <- testIntegerish(x = columns, lower = 1, min.len = 1, null.ok = TRUE)
-  colList <- testList(x = columns, len = 3)
+  colList <- testList(x = columns, len = 1)
   assert(colInt, colList)
+  if(colList) assertSubset(x = names(columns), choices = c("find"))
   rowInt <- testIntegerish(x = rows, lower = 1, min.len = 1, null.ok = TRUE)
-  rowList <- testList(x = rows, len = 3)
+  rowList <- testList(x = rows, len = 1)
   assert(rowInt, rowList)
+  if(rowList) assertSubset(x = names(rows), choices = c("find"))
   assertCharacter(x = value, len = 1, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = split, len = 1, any.missing = FALSE, null.ok = TRUE)
   assertCharacter(x = merge, len = 1, any.missing = FALSE, null.ok = TRUE)
