@@ -12,7 +12,7 @@
 #'   \emph{values} of the new variable are recorded.
 #' @param top [\code{integerish(.)}]\cr In case the variable is nested in a wide
 #'   identifying variable, specify here additionally the topmost row in which
-#'   the variable \emph{name} sits
+#'   the variable \emph{name} sits.
 #' @param unit [\code{character(1)}]\cr the unit of this variable.
 #' @param factor [\code{numeric(1)}]\cr the factor that needs to be multiplied
 #'   with the values to convert to \code{unit}, defaults to 1. For instance, if
@@ -51,11 +51,13 @@ setObsVar <- function(schema = NULL, name = NULL, columns = NULL, top = NULL,
   assertClass(x = schema, classes = "schema", null.ok = TRUE)
   assertCharacter(x = name, len = 1, any.missing = FALSE)
   colInt <- testIntegerish(x = columns, lower = 1, min.len = 1, null.ok = TRUE)
-  colList <- testList(x = columns, len = 3)
+  colList <- testList(x = columns, len = 1)
   assert(colInt, colList)
+  if(colList) assertSubset(x = names(columns), choices = c("find"))
   rowInt <- testIntegerish(x = top, lower = 1, min.len = 1, null.ok = TRUE)
-  rowList <- testList(x = top, len = 3)
+  rowList <- testList(x = top, len = 1)
   assert(rowInt, rowList)
+  if(rowList) assertSubset(x = names(top), choices = c("find"))
   assertLogical(x = relative, any.missing = FALSE, len = 1)
   assertLogical(x = distinct, any.missing = FALSE, len = 1)
   assertCharacter(x = unit, len = 1, any.missing = FALSE, null.ok = TRUE)
